@@ -34,12 +34,13 @@ using namespace google_breakpad;
 
 extern "C" int gtk_module_init (int *argc, char** argv[]);
 static bool    run_bug_buddy   (const gchar *appname, pid_t pid, const gchar *minidump_path);
-static bool    check_if_gdb    (void *callback_context);
-
+static void    check_if_gdb    ();
 
 static gchar *bugbuddy;
 
 #ifdef ENABLE_GOOGLE_BREAKPAD
+static ExceptionHandler * handler;
+
 // Callback when minidump written.
 static bool MinidumpCallback(const char *dump_path,
                              const char *minidump_id,
@@ -58,7 +59,8 @@ static bool MinidumpCallback(const char *dump_path,
   g_free (minidump_file);
   _exit(0);
 }
-#else
+#endif
+
 static void
 bugbuddy_segv_handle(int signum)
 {
@@ -78,15 +80,18 @@ bugbuddy_segv_handle(int signum)
                 _exit(1);
         }
 
-	check_if_gdb (NULL);
+	check_if_gdb ();
 
-	/* If we are here is because gdb couldn't be run.
-	 * FIXME: Show some error? */
-        
-        _exit(1);
-}
+#ifdef ENABLE_GOOGLE_BREAKPAD
+	/* If we are here is because there are no debug symbols or gdb couldn't be run.
+	 * Write the minidump anyway.
+	 */
+
+	handler->WriteMinidump ("/tmp", MinidumpCallback, NULL);
 #endif
 
+        _exit(1);
+}
 
 #define N_TRIES 3
 
@@ -251,8 +256,8 @@ run_gdb (const gchar *appname, pid_t pid)
 	return true;
 }
 
-static bool
-check_if_gdb (void *callback_context)
+static void
+check_if_gdb ()
 {
 	char mypath[255];
 	gchar *gdb;
@@ -282,15 +287,12 @@ check_if_gdb (void *callback_context)
 	readlink ("/proc/self/exe", mypath, sizeof(mypath));
 	has_debug_symbols = elf_has_debug_symbols (mypath);
 
-
 	if (bugbuddy && gdb && has_debug_symbols) {
 		res = run_bug_buddy (appname, pid, NULL);
 		if (!res)
 			_exit (1);
 		_exit(0);
 	}
-
-	return true;
 }
 		
 int
@@ -300,31 +302,11 @@ gtk_module_init (int *argc, char** argv[])
 	
 	if (bugbuddy && !g_getenv ("GNOME_DISABLE_CRASH_DIALOG")) {
 #ifdef ENABLE_GOOGLE_BREAKPAD
-        	static struct sigaction old_action;
+		/* create the handler for minidumps */
+  		handler = new ExceptionHandler ("/tmp", NULL,
+						NULL, NULL, false);
+#endif
 
-		sigaction(SIGSEGV, NULL, &old_action);
-		if (old_action.sa_handler != SIG_DFL)
-			return 0;
-
-                sigaction(SIGABRT, NULL, &old_action);
-		if (old_action.sa_handler != SIG_DFL)
-			return 0;
-
-                sigaction(SIGTRAP, NULL, &old_action);
-		if (old_action.sa_handler != SIG_DFL)
-			return 0;
-
-                sigaction(SIGFPE, NULL, &old_action);
-		if (old_action.sa_handler != SIG_DFL)
-			return 0;
-
-                sigaction(SIGBUS, NULL, &old_action);
-		if (old_action.sa_handler != SIG_DFL)
-			return 0;
-
-  		static ExceptionHandler handler("/tmp", check_if_gdb,
-						MinidumpCallback, NULL, true);
-#else
         	static struct sigaction *setptr;
         	static struct sigaction old_action;
         	struct sigaction sa;
@@ -352,7 +334,6 @@ gtk_module_init (int *argc, char** argv[])
                 sigaction(SIGBUS, NULL, &old_action);
 		if (old_action.sa_handler == SIG_DFL)
                 	sigaction(SIGBUS, setptr, NULL);
-#endif
 	}
 	return 0;
 }
