@@ -37,8 +37,10 @@
 #include <unistd.h>
 #include <sys/utsname.h>
 #include <sys/wait.h>
+#include <string.h>
 
 #include <cstdlib>
+#include <cstring>
 #include <ctime>
 
 #include "common/linux/file_id.h"
