@@ -41,31 +41,20 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <gtk/gtkbuilder.h>
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnome/libgnometypebuiltins.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdk.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
 #include <gconf/gconf-client.h>
 
-#ifdef HAVE_NETWORKMANAGER
-#include <libnm_glib.h>
-#endif
-
 #include <libsoup/soup.h>
-#include <libsoup/soup-xmlrpc-message.h>
-
-#include <sys/types.h>
-#include <signal.h>
-
-#define d(x)
 
 #define USE_PROXY_KEY 	"/system/http_proxy/use_http_proxy"
 #define PROXY_HOST_KEY	"/system/http_proxy/host"
@@ -77,31 +66,21 @@
 #define GTK_THEME_KEY	"/desktop/gnome/interface/gtk_theme"
 #define ICON_THEME_KEY  "/desktop/gnome/interface/icon_theme"
 #define DESKTOP_IS_HOME_DIR "/apps/nautilus/preferences/desktop_is_home_dir"
+#define MIN_REPORT_DETAILS_CHARS 10
 
 static GOptionData gopt_data;
 static int bug_count = 0;
 static GHashTable *apps = NULL;
 
 static const GOptionEntry options[] = {
-	{ "name",       '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.name,          N_("Name of contact"),                         N_("NAME") },
-	{ "email",      '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.email,         N_("Email address of contact"),                N_("EMAIL") },
 	{ "package",    '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.package,       N_("Package containing the program"),          N_("PACKAGE") },
-	{ "package-ver",'\0', 0, G_OPTION_ARG_STRING,   &gopt_data.package_ver,   N_("Version of the package"),                  N_("VERSION") },
 	{ "appname",    '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.app_file,      N_("File name of crashed program"),            N_("FILE") },
 	{ "pid",        '\0', 0, G_OPTION_ARG_INT,      &gopt_data.pid,           N_("PID of crashed program"),                  N_("PID") },
-	{ "core",       '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.core_file,     N_("Core file from program"),                  N_("FILE") },
 	{ "include",    '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.include_file,  N_("Text file to include in the report"),      N_("FILE") },
 	{ "minidump",   '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.minidump_file, N_("MiniDump file with info about the crash"), N_("FILE") },
-	{ "kill",       '\0', 0, G_OPTION_ARG_INT,      &gopt_data.kill,          N_("PID of the program to kill after the report"), N_("KILL") },
 	{ NULL }
 };
 
-enum {
-	NETWORK_CONNECTED,
-	NETWORK_DISCONNECTED,
-	NETWORK_UNKNOWN
-};
-
 static void fill_stderr_info (GtkBuilder *ui);
 static void fill_custom_info (BugzillaApplication *app, GtkBuilder *ui);
 static void close_callback   (GtkWidget *widget, gpointer user_data);
@@ -363,12 +342,75 @@ save_email (const char *email)
 	g_object_unref (conf_client);
 }
 
+static void
+copy_link_item_activated_cb (GtkMenuItem *menu_item,
+			     const gchar *link)
+{
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clipboard, link, -1);
+}
+
+static GtkWidget *
+build_link_menu (const gchar *link)
+{
+	GtkWidget *menu;
+	GtkWidget *item;
+	GtkWidget *image;
+
+	menu = gtk_menu_new ();
+	item = gtk_image_menu_item_new_with_mnemonic (_("Copy _Link Address"));
+	image = gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	gtk_widget_show_all (item);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (copy_link_item_activated_cb), (char *) link);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	return menu;
+}
+
+static gboolean
+link_button_press_event_cb (GtkWidget *widget,
+			    GdkEventButton *event,
+			    gpointer data)
+{
+	const gchar *link = gtk_link_button_get_uri (GTK_LINK_BUTTON (widget));
+
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		GtkWidget *popup;
+
+		popup = build_link_menu (link);
+		gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL,
+				event->button, event->time);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
 link_callback (GtkLinkButton *button, gpointer user_data)
 {
 	const gchar *link = gtk_link_button_get_uri (button);
+#if GTK_CHECK_VERSION (2,13,0)
+	GdkAppLaunchContext *context;
 
-	if (gnome_vfs_url_show (link) != GNOME_VFS_OK) {
+	context = gdk_app_launch_context_new ();
+	gdk_app_launch_context_set_screen (context,
+					   gtk_widget_get_screen (GTK_WIDGET (button)));
+	gdk_app_launch_context_set_timestamp (context,
+					      gtk_get_current_event_time ());	
+
+	if (!g_app_info_launch_default_for_uri (link,
+						G_APP_LAUNCH_CONTEXT (context),
+						NULL))
+#else
+	if (!g_app_info_launch_default_for_uri (link, NULL, NULL))
+#endif
+    {
 		char *text;
 
 		text = g_markup_printf_escaped (_("Bug Buddy was unable to view the link \"%s\"\n"), link);
@@ -376,6 +418,10 @@ link_callback (GtkLinkButton *button, gpointer user_data)
 		g_free (text);
 	}
 
+#if GTK_CHECK_VERSION (2,13,0)
+	g_object_unref (context);
+#endif
+
 	return;
 }
 
@@ -402,19 +448,19 @@ network_error (SoupMessage *msg, GtkBuilder *ui)
 		
 	dialog = gtk_message_dialog_new_with_markup (NULL,
 						      GTK_DIALOG_MODAL,
-						      GTK_MESSAGE_QUESTION,
+						      GTK_MESSAGE_WARNING,
 						      GTK_BUTTONS_YES_NO,
-						      _("<span weight=\"bold\">Network Connection Error</span>\n"
-						      "Maybe no Network Connection available.\n"
-						      "Do you want to store this report until "
-						      "a Network Connection is available?"));
+						      _("There has been a network error while sending the report. "
+							"Do you want to save this report and send it later?"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  "%s", _("Please ensure that your Internet connection is active "
+							  "and working correctly."));
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
 	if (res == GTK_RESPONSE_YES) {
 		gchar *dirname;
 		gchar *filename;
-		xmlChar *message_string;
 	       
 		dirname = g_strdup_printf ("%s/.gnome2/bug-buddy/pending_reports", g_get_home_dir ());
 		if (!g_file_test (dirname, G_FILE_TEST_IS_DIR)) {
@@ -423,11 +469,8 @@ network_error (SoupMessage *msg, GtkBuilder *ui)
 
 		filename = g_strdup_printf ("%s/%ld", dirname, (long)time (NULL));
 					
-		message_string = soup_xmlrpc_message_to_string (SOUP_XMLRPC_MESSAGE (msg));
+		save_to_file (filename, msg->request_body->data);
 
-		save_to_file (filename, (const gchar*) message_string);
-
-		xmlFree (message_string);
 		g_free (dirname);
 		g_free (filename);
 	}
@@ -473,7 +516,7 @@ all_sent (GtkBuilder *ui)
 }
 
 static void
-previous_sent (SoupMessage *msg, GtkBuilder *ui)
+previous_sent (SoupSession *session, SoupMessage *msg, GtkBuilder *ui)
 {
 	if (--bug_count == 0) {
 		all_sent (ui);
@@ -482,14 +525,13 @@ previous_sent (SoupMessage *msg, GtkBuilder *ui)
 
 
 static void
-bug_sent (SoupMessage *msg, GtkBuilder *ui)
+bug_sent (SoupSession *session, SoupMessage *msg, GtkBuilder *ui)
 {
 	GtkWidget *button;
 	GtkWidget *image;
 	char *text = NULL;
 	char *errmsg = NULL;
 	char *str = NULL;
-	char *ptr = NULL;
 	long bugid;
 	char *response;
 	char *tmp;
@@ -535,6 +577,8 @@ bug_sent (SoupMessage *msg, GtkBuilder *ui)
 		/* create a clickable link to the bug report */
 		urlbutton = gtk_link_button_new (text);
 		g_signal_connect (G_OBJECT (urlbutton), "clicked", G_CALLBACK (link_callback), NULL);
+		g_signal_connect (G_OBJECT (urlbutton), "button-press-event",
+				  G_CALLBACK (link_button_press_event_cb), NULL);
 		gtk_box_pack_end (GTK_BOX (main_vbox), urlbutton, FALSE, FALSE, 0);
 
 		gtk_widget_show (urlbutton);
@@ -554,7 +598,34 @@ bug_sent (SoupMessage *msg, GtkBuilder *ui)
 		errmsg = _("Bug Buddy has encountered an error while submitting your report "
 		           "to the Bugzilla server.  Details of the error are included below.\n\n");
 
-		if (err != NULL) {
+		if (err && err->domain == SOUP_XMLRPC_FAULT) {
+			/* see http://cvs.gnome.org/viewcvs/bugzilla-newer/Bugzilla/RPC.pm?view=markup */
+			if (err->message == NULL) {
+				text = g_strdup_printf (_("Bugzilla reported an error when trying to process your "
+							  "request, but was unable to parse the response."));
+			} else if (g_str_equal (err->message, "invalid_username")) {
+				text = g_strdup_printf (_("The email address you provided is not valid."));
+			} else if (g_str_equal (err->message, "account_disabled")) {
+				text = g_strdup_printf (_("The account associated with the email address "
+							  "provided has been disabled."));
+			} else if (g_str_equal (err->message, "product_doesnt_exist")) {
+				text = g_strdup_printf (_("The product specified doesn't exist or has been "
+							  "renamed.  Please upgrade to the latest version."));
+			} else if (g_str_equal (err->message, "component_not_valid")) {
+				text = g_strdup_printf (_("The component specified doesn't exist or has been "
+							  "renamed.  Please upgrade to the latest version."));
+			} else if (g_str_equal (err->message, "require_summary")) {
+				text = g_strdup_printf (_("The summary is required in your bug report. "
+							  "This should not happen with the latest Bug Buddy."));
+			} else if (g_str_equal (err->message, "description_required")) {
+				text = g_strdup_printf (_("The description is required in your bug report. "
+							  "This should not happen with the latest Bug Buddy."));
+			} else {
+				text = g_strdup_printf (_("The fault code returned by Bugzilla is not recognized. "
+							  "Please report the following information to "
+							  "bugzilla.gnome.org manually:\n\n%s"), err->message);
+			}
+		} else if (err) {
 			switch (err->code) {
 			case BUGZILLA_ERROR_RECV_BAD_STATUS:
 				text = g_strdup_printf (_("Server returned bad state.  This is most likely a server "
@@ -565,45 +636,6 @@ bug_sent (SoupMessage *msg, GtkBuilder *ui)
 				text = g_strdup_printf (_("Failed to parse the xml-rpc response.  Response follows:\n\n%s"), 
 				                        err->message);
 				break;
-			case BUGZILLA_ERROR_RECV_FAULT:
-				/* in this case, the error message returned is the faultCode and faultString from
-				 * the XML-RPC response, separated by a colon.  We construct our error message
-				 * based on the faultString */
-				ptr = strstr (err->message, ":");
-				if (ptr == NULL) {
-					text = g_strdup_printf (_("Bugzilla reported an error when trying to process your "
-					                          "request, but was unable to parse the response."));
-					break;
-				}
-
-				/* skip the colon */
-				ptr++;
-				
-				/* see http://cvs.gnome.org/viewcvs/bugzilla-newer/Bugzilla/RPC.pm?view=markup */
-				if (g_str_equal (ptr, "invalid_username")) {
-					text = g_strdup_printf (_("The email address you provided is not valid."));
-				} else if (g_str_equal (ptr, "account_disabled")) {
-					text = g_strdup_printf (_("The account associated with the email address "
-				                                  "provided has been disabled."));
-				} else if (g_str_equal (ptr, "product_doesnt_exist")) {
-					text = g_strdup_printf (_("The product specified doesn't exist or has been "
-				                                  "renamed.  Please upgrade to the latest version."));
-				} else if (g_str_equal (ptr, "component_not_valid")) {
-					text = g_strdup_printf (_("The component specified doesn't exist or has been "
-				                                  "renamed.  Please upgrade to the latest version."));
-				} else if (g_str_equal (ptr, "require_summary")) {
-					text = g_strdup_printf (_("The summary is required in your bug report. "
-				                                  "This should not happen with the latest Bug Buddy."));
-				} else if (g_str_equal (ptr, "description_required")) {
-					text = g_strdup_printf (_("The description is required in your bug report. "
-				                                  "This should not happen with the latest Bug Buddy."));
-				} else {
-					text = g_strdup_printf (_("The fault code returned by Bugzilla is not recognized. "
-				                                  "Please report the following information to "
-					                          "bugzilla.gnome.org manually:\n\n%s"), err->message);
-				}
-
-				break;
 			default:
 				text = g_strdup_printf (_("An unknown error occurred.  This is most likely a problem with "
 				                          "bug-buddy. Please report this problem manually at bugzilla."
@@ -640,7 +672,7 @@ set_proxy (SoupSession *session)
 	char *host;
 	int port;
 	char *proxy_uri;
-	SoupUri *uri;
+	SoupURI *uri;
 	char *username = NULL;
 	char *password = NULL;
 	
@@ -730,7 +762,7 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GtkBuild
 	char *final_text;
 	const char *email;
 	SoupSession *session;
-        SoupXmlrpcMessage *message;
+        SoupMessage *message;
 	GError *err = NULL;
 
 	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "pending-reports-check")));
@@ -792,17 +824,18 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GtkBuild
 			while (name != NULL) {
 				char *path;
 				char *contents;
+				gsize length;
 				
 				path = g_strdup_printf ("%s/%s", dirname, name);
-				if (g_file_get_contents (path, &contents, NULL, NULL)) {
-					SoupXmlrpcMessage *msg;
-					msg = soup_xmlrpc_message_new ("http://bugzilla.gnome.org/bugbuddy.cgi");
-					soup_xmlrpc_message_from_string (msg, contents);
+				if (g_file_get_contents (path, &contents, &length, NULL)) {
+					SoupMessage *msg;
+					msg = soup_message_new ("POST", "http://bugzilla.gnome.org/bugbuddy.cgi");
+					soup_message_set_request (msg, "text/xml",
+								  SOUP_MEMORY_TAKE,
+								  contents, length);
 					bug_count++;
-					soup_xmlrpc_message_persist (msg);
         				soup_session_queue_message (session, SOUP_MESSAGE (msg),
-               		                    			    (SoupMessageCallbackFn)previous_sent, ui);
-					g_free (contents);
+               		                    			    (SoupSessionCallback)previous_sent, ui);
 				}
 				g_free (path);
 				name = g_dir_read_name (dir);
@@ -814,9 +847,8 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GtkBuild
 
 	bug_count++;
 
-	soup_xmlrpc_message_persist (message);
-        soup_session_queue_message (session, SOUP_MESSAGE (message),
-                                    (SoupMessageCallbackFn)bug_sent, ui);
+        soup_session_queue_message (session, message,
+                                    (SoupSessionCallback)bug_sent, ui);
 	g_free (gdb_text);
 	g_free (details_text);
 	g_free (title);
@@ -885,6 +917,7 @@ email_domain_is_valid (const char *domain)
 {
 	char **labels;
 	char **this_label;
+	gboolean retval = FALSE;
 	
 	/* If there is no domain, there are no domain labels and the domain is
 	 * not valid. */ 
@@ -893,16 +926,22 @@ email_domain_is_valid (const char *domain)
 
 	/* Split the domain on the dot to validate labels. */
 	labels = g_strsplit (domain, ".", 0);
+	if (g_strv_length (labels) == 1) {
+		/* the domain doesn't contain any dot: it's not valid */
+		goto out;
+	}
 
 	for (this_label = labels; *this_label; this_label++) {
 		if (!email_domain_label_is_valid (*this_label)) {
-			g_strfreev (labels);
-			return FALSE;
+			goto out;
 		}
 	}
 
+	retval = TRUE;
+
+out:
 	g_strfreev (labels);
-	return TRUE;
+	return retval;
 }
 
 /* Check for *simple* email addresses of the form user@host, with checks
@@ -955,21 +994,83 @@ on_send_clicked (GtkWidget *button, gpointer data)
 	GnomeVersionInfo *gnome_version;
 	GtkRequisition requisition;
 	GtkBuilder *ui = (GtkBuilder*) data;
+	GtkWidget *details;
+	int i;
 
 	app = g_object_get_data (G_OBJECT (ui), "app");
 	gnome_version = g_object_get_data (G_OBJECT (ui), "gnome-version");
 
-	/* hide the send button immediately so that the user can't click
-	 * it more than once (this will create multiple bugs) */
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "send-button")));
+	details = GTK_WIDGET (gtk_builder_get_object (ui, "details-view"));
+	i = gtk_text_buffer_get_char_count (
+			gtk_text_view_get_buffer (GTK_TEXT_VIEW (details)));
 
-	gtk_widget_show (GTK_WIDGET (gtk_builder_get_object (ui, "progressbar")));
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "final-box")));
-	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "review-box")));
+	if (i < MIN_REPORT_DETAILS_CHARS) {
+		GtkWidget *dialog;
+		GtkWidget *button;
+		GtkWidget *icon;
 
-	gtk_widget_size_request (GTK_WIDGET (gtk_builder_get_object (ui, "main-window")), &requisition);
-	gtk_window_resize (GTK_WINDOW (gtk_builder_get_object (ui, "main-window")),
-			   requisition.width, requisition.height);
+		int i;
+
+		dialog = gtk_message_dialog_new (NULL, 
+				GTK_DIALOG_MODAL, 
+				GTK_MESSAGE_INFO,
+				GTK_BUTTONS_NONE,
+				_("The description you provided for the "
+				  "crash is very short. Are you sure you want "
+				  "to send it?"));
+
+		/* Secondary text */
+		gtk_message_dialog_format_secondary_text 
+			(GTK_MESSAGE_DIALOG (dialog),
+			 _("A short description is probably not of much help "
+			   "to the developers investigating your report. "
+			   "If you provide a better one, for instance "
+			   "specifying a way to reproduce the crash, the "
+			   "issue can be more easily resolved."));
+
+		/* Review button */
+		button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+						_("_Review description"),
+						GTK_RESPONSE_CANCEL);
+		icon = gtk_image_new_from_stock 
+				(GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON);
+		gtk_button_set_image (GTK_BUTTON (button), icon);
+		gtk_widget_show (button);
+
+		/* Send anyway button */
+		button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+						_("_Send anyway"),
+						GTK_RESPONSE_OK);
+		icon = gtk_image_new_from_stock 
+				(GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON);
+		gtk_button_set_image (GTK_BUTTON (button), icon);
+		gtk_widget_show (button);
+
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), 
+						 GTK_RESPONSE_OK);
+		
+		i = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		gtk_widget_destroy (dialog);
+
+		if (i != GTK_RESPONSE_OK)
+			return;
+	}
+
+	gtk_widget_show (GTK_WIDGET (
+			gtk_builder_get_object (ui, "progressbar")));
+	gtk_widget_hide (GTK_WIDGET (
+			gtk_builder_get_object (ui, "final-box")));
+	gtk_widget_hide (GTK_WIDGET (
+			gtk_builder_get_object (ui, "review-box")));
+
+	gtk_widget_size_request (GTK_WIDGET (
+			gtk_builder_get_object (ui, "main-window")), 
+			&requisition);
+
+	gtk_window_resize (GTK_WINDOW (
+			gtk_builder_get_object (ui, "main-window")),
+			requisition.width, requisition.height);
 			   
 	send_report (app, gnome_version, ui);
 }
@@ -1342,6 +1443,17 @@ bug_buddy_quit (GtkBuilder *ui)
 	gtk_main_quit ();
 }
 
+static gboolean
+keypress_callback (GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	if (event->keyval == GDK_Escape) {
+		close_callback (NULL, data);
+		return TRUE;
+	}
+	/* let others handle the event */
+	return FALSE;
+}
+
 static void
 close_callback (GtkWidget *widget, gpointer user_data)
 {
@@ -1656,38 +1768,6 @@ fill_stderr_info (GtkBuilder *ui)
 	g_string_free (stderr_info, TRUE);
 }
 
-
-static GdkPixbuf*
-load_icon (const char *icon)
-{
-	GdkPixbuf *pixbuf = NULL;
-
-	if (g_path_is_absolute (icon)) {
-		pixbuf = gdk_pixbuf_new_from_file_at_size (icon, 48, 48, NULL);
-	}
-	if (pixbuf == NULL) {
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						   icon, 48, 0, NULL);
-	}
-	if (pixbuf == NULL && strrchr (icon, '.') != NULL) {
-		char *name;
-		name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
-		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						   name, 48, 0, NULL);
-		g_free (name);
-	}
-
-	if (pixbuf == NULL) {
-		char *filename;
-		filename = g_strdup_printf (BUDDY_ICONDIR"/pixmaps/%s", icon);
-		pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 48, 48, NULL);
-		g_free (filename);
-	}
-
-	return pixbuf;
-}
-	
-
 int
 main (int argc, char *argv[])
 {
@@ -1735,17 +1815,22 @@ main (int argc, char *argv[])
 		gtk_builder_set_translation_domain (ui, GETTEXT_PACKAGE);
 	}
 
-	if (!ui) {
-		buddy_error (NULL, 
+	g_free (s);
+
+	if (!ui || err) {
+		buddy_error (NULL,
 			     _("Bug Buddy could not load its user interface file.\n"
 			       "Please make sure Bug Buddy was installed correctly."));
+		if (err) {
+			g_error_free (err);
+		}
 		g_object_unref (program);
 		return 0;
 	}
-	g_free (s);	
 
 	main_window = GTK_WIDGET (gtk_builder_get_object (ui, "main-window"));
 	g_signal_connect (main_window, "delete-event", G_CALLBACK (delete_callback), ui);
+	g_signal_connect (main_window, "key-press-event", G_CALLBACK (keypress_callback), ui);
 
 	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (ui, "final-box")));
 
@@ -1803,13 +1888,9 @@ main (int argc, char *argv[])
 			g_object_set_data (G_OBJECT (ui), "app", app);
 	
 			if (app->icon) {
-				GdkPixbuf *pixbuf;
-				pixbuf = load_icon (app->icon);
-				if (pixbuf) {
-					gtk_image_set_from_pixbuf (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
-			                           	   	   pixbuf);
-					g_object_unref(pixbuf);
-				}
+				gtk_image_set_from_icon_name (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
+							      app->icon,
+							      GTK_ICON_SIZE_DIALOG);
 			}
 			fill_gnome_info (app, gnome_version, ui);
 		}
@@ -1871,14 +1952,9 @@ main (int argc, char *argv[])
 		g_object_set_data (G_OBJECT (ui), "app", app);
 	
 		if (app->icon) {
-			GdkPixbuf *pixbuf;
-			
-			pixbuf = load_icon (app->icon);
-			if (pixbuf) {
-				gtk_image_set_from_pixbuf (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
-							   pixbuf);
-				g_object_unref (pixbuf);
-			}
+			gtk_image_set_from_icon_name (GTK_IMAGE (gtk_builder_get_object (ui, "app-image")),
+						      app->icon,
+						      GTK_ICON_SIZE_DIALOG);
 		}
 		fill_gnome_info (app, gnome_version, ui);
 		fill_custom_info (app, ui);
